%option noyywrap
%{
/* ukljucujmo potrebna zaglavlja */
#include <stdlib.h>
#include <string.h>
#include "y.tab.h"

extern void yyerror(char *);

%}

DIGIT [0-9]
LETTER [a-zA-Z_]

%%
  /* Prvo navodimo kljucne reci interpretera */
string                { return TYPE_STRING; }
print                 { return PRINT; }
reverse               { return REVERSE; }
sort                  { return SORT; }
length                { return LENGTH; }
substring             { return SUBSTRING; }
upper                 { return UPPER; }
lower                 { return LOWER; }
{LETTER}({LETTER}|{DIGIT})*             {
                        /* identifikatori se sastoje samo od slova i donje crte 
                           alociramo mesto za u kojem cemo da pamtimo ime identifikatora + 1 bajt za \0 */
                        yylval.p = malloc(strlen(yytext)+1);
                        if (yylval.p==NULL)
                          yyerror("Greska pri alokaciji memorije");
                        strcpy(yylval.p, yytext);
                        return ID;
                      }
\"([^\"]|[\\].)*\"    { 
                        /* niske pocinju " i u sebi mogu imati bilo sta, osim " ispred kojih nije \
                           alociramo mesto za smestanje niske, ali bez mesta za navodnike */
                        yylval.p = malloc(strlen(yytext)-1);
                        if (yylval.p==NULL)
                          yyerror("Greska pri alokaciji memorije");
                        strncpy(yylval.p, yytext+1, strlen(yytext)-2);
                        yylval.p[strlen(yytext)-2] = '\0';
                        return STRING; }
{DIGIT}+              {
                        yylval.n = atoi(yytext);
                        return NUM;
                      }
"+="                  { return CONCATENATE; }
==                    { return EQUAL; }
[+*\[\];()=\<\>,]     { return *yytext; }
[ \t\n]               {}
.                     {
                        char poruka[21];
                        sprintf(poruka, "Nepoznat karakter: %c", *yytext);
                        yyerror(poruka);
                      }
%%

