%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "y.tab.h"

  /* deklaracija fje koja vrsi leksicku analizu */  
  extern int yylex();

  /* fja za obradu gresaka */  
  void yyerror(char *poruka) {
    fprintf(stderr, "%s\n", poruka);
    exit(EXIT_FAILURE);
  }

  /* fja koja prosledjenu nisku pretvara u onu koja je pogldna za ispis
     ako negde u niski imamo karakter \ iza koga sledi karakter n, t ili "
     ta dva karaktera zamenjujemo sa jednim karakterom */
  void interpolacija(char *s) {
    char *i=s;
    for (;*i; i++) {
      if (*i == '\\') {
	switch(*(i+1)) {
	case 'n':
	  *i = '\n';
	  break;
	case 't':
	  *i = '\t';
	  break;
	case '"':
	  *i = '"';
	  break;
	}
	strcpy(i+1, i+2);
      }
    }
  }

  /* zbog faze razvoja */
#define YYDEBUG 1
  
  /* maksimalni broj promenljivih */
#define MAX 32

  /* struktura koja predstavlja promenljivu */
  typedef struct {
    char *ime;
    char *vrednost;
  } promenljiva;

  /* tablica simbola */
  promenljiva promenljive[MAX];
  int trenutni_broj_promenljivih = 0;

  /*fja koja proverava da li se neka promenljiva nalazi u tablici simbola */
  promenljiva* definisana(char *ime) {
    int i=0;
    for (;i<trenutni_broj_promenljivih; i++)
      if(strcmp(promenljive[i].ime, ime)==0)
	return &promenljive[i];
    return NULL;
  }

  /* Fja poredjenja koja je potrebna fju qsort */
  int poredi_po_slovu(void *a, void *b) {
    char _a = *(char *)a;
    char _b = *(char *)b;
    return _a-_b;
  }

  %}

%union {
  char *p;
  int n;
}

%left '+'
%left '*'
%left '[' ']'

%token <n> NUM PRINT TYPE_STRING CONCATENATE EQUAL REVERSE SORT LENGTH SUBSTRING UPPER LOWER
%token <p> STRING ID

%type <p> Izraz

%%
Program: Program Naredba
| 
;


Naredba: PRINT '(' Izraz ')' ';' {
  /* ispisujemo string i oslobadjamo memoriju koju je on zauzimao */
  printf("%s", $3);
  free($3);
 }

| TYPE_STRING ID '=' Izraz ';' {
  /* Ako smo dostigli maksimalan broj promenljivih */
  if(trenutni_broj_promenljivih==MAX)
    yyerror("Dostigli ste maximalan broj promenljivih");
  /* Ako je promenljiva vec definisana */
  if (definisana($2)!=NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s je vec definisana", $2);
    yyerror(poruka);
  }
  /* Inace uvodimo novu promenljivu u tablicu simbola, samo pokazivace preusmerivsi */
  promenljive[trenutni_broj_promenljivih].ime = $2;
  promenljive[trenutni_broj_promenljivih++].vrednost = $4;
 }

| TYPE_STRING ID ';' {
  /* Ako smo dostigli maksimalan broj promenljivih */
  if(trenutni_broj_promenljivih==MAX)
    yyerror("Dostigli ste maximalan broj promenljivih");
  /* Ako je promenljiva vec definisana */
  if (definisana($2)!=NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s je vec definisana", $2);
    yyerror(poruka);
  }
  /* Inace uvodimo novu promenljivu u tablicu simbola */
  promenljive[trenutni_broj_promenljivih].ime = $2;
  promenljive[trenutni_broj_promenljivih].vrednost = malloc(sizeof(char));
  if (promenljive[trenutni_broj_promenljivih++].vrednost==NULL)
    yyerror("Greska pri alokaciji memorije");
  promenljive[trenutni_broj_promenljivih-1].vrednost[0] = '\0';
 }

| ID CONCATENATE Izraz ';' {
  promenljiva *tmp = definisana($1);
  /* Ako promenljiva nije definisana prekidamo program */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $1);
    yyerror(poruka);
  }
  /* a inace na nju nadovezujemo nisku */
  tmp->vrednost = realloc(tmp->vrednost, strlen(tmp->vrednost)+strlen($3)+1);
  if (tmp->vrednost==NULL)
    yyerror("Greska pri alokaciji memorije");
  strcat(tmp->vrednost, $3);
  /* oslobadjamo nepotrebnu memoriju */
  free($1);
  free($3);
 }

| ID '=' Izraz ';' {
  promenljiva *tmp = definisana($1);
  /* Ako promenljiva nije definisana prekidamo program */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $1);
    yyerror(poruka);
  }
  /* a inace je predefinisemo */
  free(tmp->vrednost);
  tmp->vrednost = $3;
  /* oslobadjamo nepotrebnu memoriju */
  free($1);
 }

| Izraz '<' Izraz ';' {
  strcmp($1,$3)<0 ? printf("true\n") : printf("false\n");
  free($1);
  free($3);
}

| Izraz '>' Izraz ';' {
  strcmp($1,$3)>0 ? printf("true\n") : printf("false\n");
  free($1);
  free($3);
}

| Izraz EQUAL Izraz ';' {
  strcmp($1,$3)==0 ? printf("true\n") : printf("false\n");
  free($1);
  free($3);
}

| REVERSE '(' ID ')' ';' {
  promenljiva *tmp = definisana($3);
  /* ako promenljiva nije definisana */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $3);
    yyerror(poruka);
  }
  int i, j;
  for (i=0, j=strlen(tmp->vrednost)-1; i<j; i++, j--) {
    char slovo;
    slovo = tmp->vrednost[i];
    tmp->vrednost[i] = tmp->vrednost[j];
    tmp->vrednost[j] = slovo;
  }
}

| SORT '(' ID ')' ';' {
  promenljiva *tmp = definisana($3);
  /* ako promenljiva nije definisana */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $3);
    yyerror(poruka);
  }
  qsort(tmp->vrednost, strlen(tmp->vrednost), sizeof(char), &poredi_po_slovu);
}

| LENGTH '(' Izraz ')' ';' {
  printf("%d", strlen($3));
  free($3);
}

| SUBSTRING '(' Izraz ',' Izraz ')' ';' {
  if (strstr($3, $5)==NULL)
    printf("false\n");
  else
    printf("true\n");
  /* Oslobadjamo nepotrebnu memoriju */
  free($3);
  free($5);
}

| UPPER '(' ID ')' ';' {
  promenljiva *tmp = definisana($3);
  /* ako promenljiva nije definisana */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $3);
    yyerror(poruka);
  }
  int i= 0;
  for (;i<strlen(tmp->vrednost); i++)
    tmp->vrednost[i] = toupper(tmp->vrednost[i]);
}

| LOWER '(' ID ')' ';' {
  promenljiva *tmp = definisana($3);
  /* ako promenljiva nije definisana */
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $3);
    yyerror(poruka);
  }
  int i= 0;
  for (;i<strlen(tmp->vrednost); i++)
    tmp->vrednost[i] = tolower(tmp->vrednost[i]);
}
;


Izraz: Izraz '+' Izraz {
  /* alociramo mesta za nisku koja se dobija nadovezivanjem */
  $$ = malloc(strlen($1)+strlen($3)+1);
  if ($$==NULL)
    yyerror("Greska pri alokaciji memorije");
  /* u nju smestimo prvo prvu nisku, pa zatim drugu */
  strcpy($$, $1);
  strcpy($$+strlen($1), $3);
  /* oslobodimo nepotrebne resurse */
  free($1);
  free($3);
 }

| Izraz '*' NUM {
  unsigned i=0, duzina = strlen($1);
  $$ = malloc(duzina*$3+1);
  if ($$==NULL)
    yyerror("Greska pri alokaciji memorije");
  for (; i<$3; i++) {
    strcpy($$+i*duzina, $1);
  }
  free($1);
 }

| Izraz '[' NUM ',' NUM ']' {
  /* ako indeksi nisu dobri, prekidamo program */
  assert($3<strlen($1) && $3+$5<=strlen($1));
  /* alociramo memoriju */
  $$ = malloc($5+1);
  if ($$==NULL)
    yyerror("Greska pri alokaciji memorije");
  /* kopiramo odgovarajuca slova */
  strncpy($$, $1+$3, $5);
  $$[$5] = '\0';
  /* oslobadjamo nepotrebno */
  free($1);
 }

| '(' Izraz ')' {
  /* ovde mozemo samo da preusmerimo pokazivace */
  $$ = $2;
 }

| STRING  {
  /* u niski zamenjujemo '\''n' sa '\n' i slicno za \t i \" */
  interpolacija($1);
  /* preusmerimo pokazivace */
  $$ = $1;
 }

| ID {
  promenljiva *tmp = definisana($1);
  if (tmp==NULL) {
    char poruka[128];
    sprintf(poruka, "Promenljiva %s nije definisana", $1);
    yyerror(poruka);
  }
  /* alociramo memoriju za nisku */
  $$ = malloc(strlen(tmp->vrednost)+1);
  if ($$==NULL)
    yyerror("Greska pri alokaciji memorije");
  strcpy($$, tmp->vrednost);
  }
;


%%

int main() {
  //yydebug = 1;
  return yyparse();
}
